#!/usr/bin/env python

import csv
from optparse import OptionParser, OptionGroup
import os
import re
from subprocess import Popen, PIPE
import sys
import time

result_pat = re.compile("result=(\d+)")

class InvalidOptions(Exception):
    pass

def validate_options():
    global options, args, parser, ssh_cmd, sshni

    if (not options.srcfiler) or (not options.user) or (not options.infofile) \
            or (not options.destvol) or (not options.srcvol):
        
        print "\nSome of the required options are missing.\n"
        raise InvalidOptions

    if options.destfiler and ((not options.destuser) or (not options.destpass)):
        print "\nCredentials for destination filer %s must be passed\n" % options.destfiler
        raise InvalidOptions

    if options.destvol.startswith("/vol/"):
        options.destvol_abs = options.destvol
        options.destvol = options.destvol[5:]
    else:
        options.destvol_abs = "/vol/" + options.destvol

    if options.srcvol.startswith("/vol/"):
        options.srcvol_abs = options.srcvol
        options.srcvol = options.srcvol[5:]
    else:
        options.srcvol_abs = "/vol/" + options.srcvol

    if options.password:
        ssh_cmd = 'SSHNI_PASSWORD="<FA_SRCPASS>" %s' % sshni
    else:
        ssh_cmd = "ssh -oBatchMode=yes"

def process_options():
    global options, args, parser

    parser = OptionParser()

    command_group = OptionGroup(parser, "Operational Commands", 
                                "At least one of these needs to be used " +
                                "in order for the script to do anything meaningful.")

    command_group.add_option("--copy", action="store_true", default=False, help="Do ndmp copy.")
    command_group.add_option("--delsource", action="store_true", default=False,
                             help="Delete files from source filer.")

    parser.add_option_group(command_group)

    required_group = OptionGroup(parser, "Required options (yes, an oxymoron)")
    required_group.add_option("--srcfiler", help="Source Filer.")
    required_group.add_option("--user", help="User name for source filer.")

    required_group.add_option("--infofile", help="File containing information about paths to be processed." +
                              " Each row in the CSV file should contain directory path, " +
                              "file name, and type as 4th, 5th, and 6th components respectively. " +
                              "Only entries with type as 'FILE' are processed. " +
                              "Blank lines and lines starting with '#' are ignored.")

    required_group.add_option("--destvol", help='Destination volume (with or without the prefix "/vol/").')
    required_group.add_option("--srcvol", help='Source volume (with or without the prefix "/vol/").')
    parser.add_option_group(required_group)

    parser.add_option("-n", action="store_true", dest="trialrun", default=False, 
                      help="Do not run actual commands but only display them.")
    parser.add_option("--destfiler", help="Destination Filer. If not given, source filer will be used" + 
                      " as destination as well.")
    parser.add_option("--destuser", help="User for destination Filer.")
    parser.add_option("--destpass", default="", help="Password for destination Filer and user.")

    parser.add_option("--password", default="", 
                      help="Password for the given source filer and user. If not given," +
                      " it is expected that a trust relationship with the source filer has been configured.")

    parser.add_option("--nosnap", action="store_true", default=False, 
                      help="Do not create a snapshot." +
                      " By default, a snapshot is created and is used in the source file paths." )

    (options, args) = parser.parse_args()

    validate_options()

class CsvFileReader(object):
    def __init__(self, filename):
        self.csv_f = open(filename, "rb")

    def __iter__(self):
        return self

    def next(self):
        while True:
            line = self.csv_f.next().strip()
            if not line:
                continue
            if line.startswith("#"):
                continue

            return line

def get_file_paths():
    for row in csv.reader(CsvFileReader(options.infofile)):
        entry_type = row[5]
        if entry_type.lower() != "file":
            continue

        dirname = row[3]
        if dirname == "/":
            dirname = ""
        basename = row[4]

        yield dirname, basename

def run_command(cmd):
    runcmd, sanitized_cmd = replace_passwords(cmd)
    
    if options.trialrun:
        print(sanitized_cmd)
    else:
        os.system(runcmd)

def create_snapshot(snapname):
    snap_cmd = "%s %s@%s snap create %s %s" % (ssh_cmd, options.user, options.srcfiler, options.srcvol, snapname)
    run_command(snap_cmd)

def delete_snapshot(snapname):
    if not snapname:
        return

    snap_cmd = "%s %s@%s snap delete %s %s" % (ssh_cmd, options.user, options.srcfiler, options.srcvol, snapname)
    run_command(snap_cmd)

def build_ndmpcopy_cmd():
    global snapname

    if snapname:
        fileroot = "%s/.snapshot/%s" % (options.srcvol_abs, snapname)
    else:
        fileroot = options.srcvol_abs

    destfiler_cred = ""
    if options.destfiler:
        destfiler_cred = '-da "%s:<FA_DESTPASS>"' % options.destuser

    copy_cmd = '%s %s@%s \'ndmpcopy %s "%s%%(dirname)s/%%(basename)s" ' % (ssh_cmd, options.user, options.srcfiler, 
                                                                        destfiler_cred, fileroot)
    
    if options.destfiler:
        copy_cmd += '"%s:' % options.destfiler
    else:
        copy_cmd += '"'

    copy_cmd += '%s/%s%%(dirname)s" ; result -l\'' % (options.destvol_abs, options.srcvol)

    return copy_cmd

def replace_passwords(cmd):
    runcmd = cmd.replace("<FA_SRCPASS>", options.password).replace("<FA_DESTPASS>", options.destpass)

    sanitized_cmd = cmd.replace("<FA_SRCPASS>", "***").replace("<FA_DESTPASS>", "***")

    return runcmd, sanitized_cmd

def delete_file(dirname, basename):
    delete_cmd = '%s %s@%s \'priv set advanced ; rm "%s%%s/%%s"\'' % \
        (ssh_cmd, options.user, options.srcfiler, options.srcvol_abs)

    run_command(delete_cmd % (dirname, basename))

def ndmp_copy(copy_cmd):
    runcmd, sanitized_cmd = replace_passwords(copy_cmd)
    if options.trialrun:
        print sanitized_cmd
        return True

    output = Popen(runcmd, stdout=PIPE, shell=True).communicate()[0]
    m = result_pat.search(output)
    if not m:
        print 'Could not find "result" in ndmpcopy output:\n(%s)\n, cmd (%s), will treat the operation as failed' % (output, sanitized_cmd)
        return False

    status = int(m.group(1))
    if status == 255:
        print "Cmd (%s) succeeded" % (sanitized_cmd)
        return True
    else:
        print "Cmd (%s) failed, status (%d), output:\n(%s)\n" % (sanitized_cmd, status, output)
        return False
            
def process():
    if (not options.copy) and (not options.delsource):
        return

    copy_cmd = build_ndmpcopy_cmd()
    for dirname, basename in get_file_paths():
        copied = True

        if options.copy:
            copied = ndmp_copy(copy_cmd % {'dirname': dirname, 'basename': basename})

        if options.delsource:
            if copied:
                delete_file(dirname, basename)
            else:
                print "Will not be deleting file (%s/%s)" % (dirname, basename)

def main():
    global snapname, sshni

    sshni = os.path.join(os.path.dirname(sys.argv[0]), "sshni")

    try:
        process_options()
    except InvalidOptions:
        parser.print_help()
        return 

    snapname = None
    if not options.nosnap:
        snapname = "snap_%d" % int(time.time())
        create_snapshot(snapname)

    try:
        process()
    finally:
        delete_snapshot(snapname)

if __name__ == "__main__":
    main()
