#!/bin/bash

LUCENEDIR="/opt/virgo/repository/ecx-usr/com.syncsort.dp.xsb.lucene.properties"
JARSRC="/opt/virgo/repository/ecx-usr/com.syncsort.dp.xsb.lucene-1.0.1-SNAPSHOT.jar"
JARFILE="lucene-core-4.2.1.jar"

LUCIDXPATH=`grep lucene.repository ${LUCENEDIR} | cut -d "=" -f 2`
LUCFILEPATH=`grep lucene.file.repository ${LUCENEDIR} | cut -d "=" -f 2`

usage(){
  echo "Usage:"
  echo "$1 [<OPTIONS>] [hiobj|file]"
  echo "  -h print usage"
  echo "  -d diagnose if there is a corruption"
  echo "  -u clear the write lock due to an abnormal shutdown"
  echo "  -f attempt to make a fix to a corruption. "
  echo "     This may result partial data loss. It is recommented to first have "
  echo "     the database backed up with -b option"
  echo "  -b <path> backup the database to the given directory"
  echo "  -r <path> restore the database from the given directory"
  echo ""
  echo "[hiobj|file]"
  echo "  hiobj: operate on hi level object database"
  echo "  file:  operate on file database"
  echo "  (default): operate on both databases"
  echo ""
  echo "Please note all operations to lucene database should be performed when virgo service is down!"
  echo ""
}


backup(){
    declare lucdir
    declare lucdest

    lucdir=`echo $1 | rev | cut -d "/" -f 1 | rev`
    lucdest=$2/${lucdir}
    echo "Backing up $1 to ${lucdest}"
    if [ ! -d $2 ]; then
       mkdir -p $2
    fi
    cp -r -p $1 ${lucdest}

    if [ $? -ne 0 ]; then
      echo "backup command failed; exiting"
      return 1
    fi

    return 0
}

restore() {
    declare lucdir
    declare bklucsrc

    lucdir=`echo $1 | rev | cut -d "/" -f 1 | rev`
    bklucsrc=$2/${lucdir}

    if [ ! -d ${bklucsrc} ]; then
       echo "${bklucsrc} does not exist"
       return 1
    fi

    echo "Restoring ${bklucsrc} to $1"
    rm -f $1/*
    cp -r -p ${bklucsrc}/* $1
}

unlock(){
    rm -f "$1/write.lock"
}

fixlucene(){
    declare lucidxpath
    declare lucfilepath

    jar xf ${JARSRC} ${JARFILE}
    if [ $? -ne 0 ]; then
      echo "Current directory must have write permission"
      return 1
    fi

    java -ea:org.apache.lucene -cp ${JARFILE} org.apache.lucene.index.CheckIndex $1 $2

    rm -f ${JARFILE}
}


OPTION_FIX=0
OPTION_HELP=0
OPTION_DIAG=0
OPTION_UNLOCK=0
OPTION_BACKUP=0
OPTION_RESTORE=0
DB_HI=0
DB_FILE=0

cmd=$0
# Parse command line options.
while getopts fdub:r:help OPT; do
    case "${OPT}" in
        f)  # fix corrupted indicis
            OPTION_FIX=1
            ;;
        h) OPTION_HELP=1
            ;;
        e|l|p) ;;
        d) OPTION_DIAG=1
            ;;
        u) OPTION_UNLOCK=1
            ;;
        b) OPTION_BACKUP=1
            BACKUP_DIR=$OPTARG
            ;;
        r) OPTION_RESTORE=1
            BACKUP_DIR=$OPTARG
            ;;
        \?) # parsing error 
            OPTION_HELP=1
            ;;
    esac
done

# Parse commandline arguments
n=1
while [ $# -gt 0 ]; do
    if [ $n -lt $OPTIND ]; then
      let n=$n+1
      shift
    else
      break;
    fi
done

if [ $# -gt 0 ]; then
    if [ "$1" == "hiobj" ]; then
        DB_HI=1
    elif [ "$1" == "file" ]; then
        DB_FILE=1    
    else
        echo "$1 is unknown"
        exit 1
    fi

    if [ $# -gt 1 ]; then
        if [ "$2" == "hiobj" ]; then
            DB_HI=1
        elif [ "$2" == "file" ]; then 
            DB_FILE=1
        else
            echo "$2 is unknown"
            exit 1
        fi
    fi
else
    DB_HI=1
    DB_FILE=1
fi


# Print usage help
if [ ${OPTION_HELP} -eq 1 ]; then
  usage $cmd
  exit 0
fi

# Validation
if [ ${OPTION_BACKUP} -eq 1 ] && [ ${OPTION_RESTORE} -eq 1 ]; then
  echo "-b and -r options conflict!"
  exit 1
fi

# Process BACKUP command
if [ ${OPTION_BACKUP} -eq 1 ]; then
  if [ ${DB_HI} -eq 1 ]; then
      backup ${LUCIDXPATH} ${BACKUP_DIR}
      if [ $? -ne 0 ]; then
          exit 1
      fi
  fi
  if [ ${DB_FILE} -eq 1 ]; then
      backup ${LUCFILEPATH} ${BACKUP_DIR}
      if [ $? -ne 0 ]; then
          exit 1
      fi
  fi
fi

# Process RESTORE command
if [ ${OPTION_RESTORE} -eq 1 ]; then
  if [ ${DB_HI} -eq 1 ]; then
      restore ${LUCIDXPATH} ${BACKUP_DIR}
      if [ $? -ne 0 ]; then
          exit 1
      fi
      OPTION_UNLOCK=1  #just in case
  fi
  if [ ${DB_FILE} -eq 1 ]; then
      restore ${LUCFILEPATH} ${BACKUP_DIR}
      if [ $? -ne 0 ]; then
          exit 1
      fi
      OPTION_UNLOCK=1 #just in case
  fi
fi

# Process DIAGNOSE or FIX command
if [ ${OPTION_FIX} -eq 1 ]; then
  if [ ${OPTION_BACKUP} -eq 0 ]; then
    while true; do
      read -p "It is strongly recommended you have backed up lucene DBs. Do you wish to continue (y/n)?" yn
      case $yn in
        [Yy]* ) 
             if [ ${DB_HI} -eq 1 ]; then
                 fixlucene ${LUCIDXPATH} "-fix"
             fi
             if [ ${DB_FILE} -eq 1 ]; then
                 fixlucene ${LUCFILEPATH} "-fix"
             fi
             break;;
        [Nn]* ) exit 1; break;;
      esac
    done
  fi
elif [ ${OPTION_DIAG} -eq 1 ]; then
  if [ ${DB_HI} -eq 1 ]; then
      fixlucene ${LUCIDXPATH}
  fi
  if [ ${DB_FILE} -eq 1 ]; then
      fixlucene ${LUCFILEPATH}
  fi
fi

# Process UNLOCK command
if [ ${OPTION_UNLOCK} -eq 1 ]; then
  if [ ${DB_HI} -eq 1 ]; then
      unlock ${LUCIDXPATH}
  fi
  if [ ${DB_FILE} -eq 1 ]; then
      unlock ${LUCFILEPATH}
  fi
fi

exit 0
